#!/usr/bin/env python3
#coding: utf-8
from flask import Flask, render_template, Markup, request, redirect, url_for, make_response
from classes import task
from funcs import dumptasks, gettasks, cattasks, deltask, findtask, addtask, mktaskdir, exporttasks, getthemes, switchstatus, todotasks, donetasks
from random import choice

#!---------- sq_todo.py ----------
# My todo list
#-----------------------------!


#----------! MAIN
app = Flask(__name__)
mktaskdir()
#Theme variable will be made global in every flask function
#css path will then be deducted

@app.route('/', methods=['GET'])
def render():
    #Does the user have a theme ?
    #theme list
    themes = getthemes()

    #Setting default theme if the user does not have one...
    if request.cookies.get('csslink') is None:
        csslink = themes[0]
        resp = make_response(render_template("homepage.html", ntodo = todotasks(gettasks()), ndone = donetasks(gettasks()), csslink = csslink))
        resp.set_cookie("csslink", csslink)
    #...or using their preferred theme if do have one
    else:
        csslink = request.cookies.get('csslink')
        resp = make_response(render_template("homepage.html", ntodo = todotasks(gettasks()), ndone = donetasks(gettasks()), csslink = csslink))

    #Delete has been clicked
    if request.args.get("delete") is not None:
        todelete = request.args.get("delete")
        deltask(int(todelete))

    #Edit has been clicked
    elif request.args.get("edit") is not None:
        toedit = request.args.get("edit")
        return redirect(url_for('edit', tasknumber=toedit))

    #Switch theme has been clicked
    elif request.args.get("switchpls") is not None:
        switchpls = request.args.get("switchpls")
        #Clicking "Change theme" switches to a random (other) theme
        current = csslink
        while csslink == current:
            csslink = choice(themes)

        #Commiting new theme, setting cookie for it, return template
        #resp = make_response(render_template("homepage.html", nr = cattasks(gettasks()), csslink = csslink))
        resp = make_response(redirect(request.path,code=302))
        resp.set_cookie('csslink', csslink)
        return resp

    #Read has been clicked
    elif request.args.get("toread") is not None:
        toread = request.args.get("toread")
        return redirect(url_for('readmode', task=(int(toread))))

    #Task is done or todo (switched)
    elif request.args.get("toswitch") is not None:
        toswitch = request.args.get("toswitch")
        mytask = findtask(int(toswitch))
        print(mytask)
        mytask = switchstatus(mytask)
        deltask(toswitch)
        addtask(mytask)
        resp = make_response(render_template("homepage.html", ntodo = todotasks(gettasks()), ndone = donetasks(gettasks()), csslink = csslink))
    else:
        resp = make_response(render_template("homepage.html", ntodo = todotasks(gettasks()), ndone = donetasks(gettasks()), csslink = csslink))

    return resp

#Export mode
@app.route('/export', methods=['GET'])
def rawtasks():
    #No theme in export
    return render_template("export.html", rawtasks = exporttasks())

#Read mode
@app.route('/readmode', methods=['GET','POST'])
def readmode():
    #theme
    csslink = request.cookies.get('csslink')


    #Render page
    if request.method == 'GET':
        tasknumber = request.args.get("task")
        mytask = findtask(int(tasknumber))
    return render_template("read.html", task=mytask.flaskrender(), csslink = csslink)


#Edition mode
@app.route('/edit', methods=['GET', 'POST'])
def edit():
    import time
    #theme
    csslink = request.cookies.get('csslink') 

    #Render edition page (Landing)
    if request.method == "GET":
        tasknumber = request.args.get("tasknumber")
        mytask = findtask(int(tasknumber))
        #Is task done ? 
        if mytask.donemark == "✅":
            todoselected = ""
            doneselected = "selected"
        else:
            todoselected = "selected"
            doneselected = ""

        return render_template("edit.html", tasknumber=tasknumber, ntitle=mytask.title, ntext=mytask.text, csslink = csslink, priority=mytask.priority, taskdonemark=mytask.donemark, todoselected=todoselected, doneselected=doneselected)

    #Handling post
    if request.method == "POST":
        #Load values
        tasktitle = request.form['title']
        tasktext = request.form['text']
        tasknumber = int(request.form['tasknumber'])
        taskpriority = int(request.form['priority'])
        #Treat the "done" value which is annoying
        doneselected = request.form['donemark']
        if doneselected == "✅":
            taskdone = True
        else:
            taskdone = False
        deltask(tasknumber)
        rightnow = int(time.time())
        newtask = task(createtime=tasknumber, modtime=rightnow, title=tasktitle, text=tasktext, done=taskdone, priority=taskpriority)
        addtask(newtask)
        return render_template("read.html", task=newtask.flaskrender(), csslink = csslink)


#Basic route, allows task creation
@app.route('/', methods=['POST'])
def homepage():
    import time

    #theme
    csslink = request.cookies.get('csslink')

    #New task
    try:
        tasktitle = request.form['title']
        tasktext = request.form['text']
        print("PING")
        priority = request.form['priority']

        print(f"Got new task with prio {priority}")

        rightnow = int(time.time())
        newtask = task(createtime=rightnow, modtime=rightnow, title=tasktitle, text=tasktext, done=False, priority=int(priority))
        addtask(newtask)
    except:
        pass
    return render_template("homepage.html", ntodo = todotasks(gettasks()), ndone = donetasks(gettasks()), csslink = csslink)

if __name__ == '__main__':
    app.run(host="0.0.0.0")
